<?php
use \Chibi\Sql as Sql;
use \Chibi\Database as Database;

final class UserModel extends AbstractCrudModel
{
	public static function getTableName()
	{
		return 'user';
	}

	protected static function saveSingle($user)
	{
		$user->validate();

		Database::transaction(function() use ($user)
		{
			self::forgeId($user);

			$bindings = [
				'name' => $user->getName(),
				'pass_salt' => $user->getPasswordSalt(),
				'pass_hash' => $user->getPasswordHash(),
				'staff_confirmed' => $user->isStaffConfirmed() ? 1 : 0,
				'email_unconfirmed' => $user->getUnconfirmedEmail(),
				'email_confirmed' => $user->getConfirmedEmail(),
				'join_date' => $user->getJoinTime(),
				'last_login_date' => $user->getLastLoginTime(),
				'access_rank' => $user->getAccessRank()->toInteger(),
				'settings' => $user->getSettings()->getAllAsSerializedString(),
				'banned' => $user->isBanned() ? 1 : 0,
				];

			$stmt = (new Sql\UpdateStatement)
				->setTable('user')
				->setCriterion(new Sql\EqualsFunctor('id', new Sql\Binding($user->getId())));

			foreach ($bindings as $key => $val)
				$stmt->setColumn($key, new Sql\Binding($val));

			Database::exec($stmt);
		});

		return $user;
	}

	protected static function removeSingle($user)
	{
		Database::transaction(function() use ($user)
		{
			$binding = new Sql\Binding($user->getId());

			$stmt = new Sql\DeleteStatement();
			$stmt->setTable('post_score');
			$stmt->setCriterion(new Sql\EqualsFunctor('user_id', $binding));
			Database::exec($stmt);

			$stmt->setTable('favoritee');
			Database::exec($stmt);

			$stmt->setTable('user');
			$stmt->setCriterion(new Sql\EqualsFunctor('id', $binding));
			Database::exec($stmt);

			$stmt = new Sql\UpdateStatement();
			$stmt->setTable('comment');
			$stmt->setCriterion(new Sql\EqualsFunctor('commenter_id', $binding));
			$stmt->setColumn('commenter_id', new Sql\NullFunctor());
			Database::exec($stmt);

			$stmt = new Sql\UpdateStatement();
			$stmt->setTable('post');
			$stmt->setCriterion(new Sql\EqualsFunctor('uploader_id', $binding));
			$stmt->setColumn('uploader_id', new Sql\NullFunctor());
			Database::exec($stmt);
		});
	}



	public static function getByName($key)
	{
		$ret = self::tryGetByName($key);
		if (!$ret)
			throw new SimpleNotFoundException('Invalid user name "%s"', $key);
		return $ret;
	}

	public static function tryGetByName($key)
	{
		$stmt = new Sql\SelectStatement();
		$stmt->setColumn('*');
		$stmt->setTable('user');
		$stmt->setCriterion(new Sql\NoCaseFunctor(new Sql\EqualsFunctor('name', new Sql\Binding(trim($key)))));

		$row = Database::fetchOne($stmt);
		return $row
			? self::spawnFromDatabaseRow($row)
			: null;
	}

	public static function getByEmail($key)
	{
		$ret = self::tryGetByEmail($key);
		if (!$ret)
			throw new SimpleNotFoundException('Invalid user e-mail "%s"', $key);
		return $ret;
	}

	public static function tryGetByEmail($key)
	{
		$key = trim($key);

		$stmt = new Sql\SelectStatement();
		$stmt->setColumn('*');
		$stmt->setTable('user');
		$stmt->setCriterion((new Sql\DisjunctionFunctor)
			->add(new Sql\NoCaseFunctor(new Sql\EqualsFunctor('email_unconfirmed', new Sql\Binding($key))))
			->add(new Sql\NoCaseFunctor(new Sql\EqualsFunctor('email_confirmed', new Sql\Binding($key)))));

		$row = Database::fetchOne($stmt);
		return $row
			? self::spawnFromDatabaseRow($row)
			: null;
	}



	public static function updateUserScore($user, $post, $score)
	{
		Database::transaction(function() use ($user, $post, $score)
		{
			$post->removeCache('score');
			$stmt = new Sql\DeleteStatement();
			$stmt->setTable('post_score');
			$stmt->setCriterion((new Sql\ConjunctionFunctor)
				->add(new Sql\EqualsFunctor('post_id', new Sql\Binding($post->getId())))
				->add(new Sql\EqualsFunctor('user_id', new Sql\Binding($user->getId()))));
			Database::exec($stmt);
			$score = intval($score);
			if (abs($score) > 1)
				throw new SimpleException('Invalid score');
			if ($score != 0)
			{
				$stmt = new Sql\InsertStatement();
				$stmt->setTable('post_score');
				$stmt->setColumn('post_id', new Sql\Binding($post->getId()));
				$stmt->setColumn('user_id', new Sql\Binding($user->getId()));
				$stmt->setColumn('score', new Sql\Binding($score));
				Database::exec($stmt);
			}
		});
	}

	public static function addToUserFavorites($user, $post)
	{
		Database::transaction(function() use ($user, $post)
		{
			$post->removeCache('fav_count');
			self::removeFromUserFavorites($user, $post);
			$stmt = new Sql\InsertStatement();
			$stmt->setTable('favoritee');
			$stmt->setColumn('post_id', new Sql\Binding($post->getId()));
			$stmt->setColumn('user_id', new Sql\Binding($user->getId()));
			$stmt->setColumn('fav_date', time());
			Database::exec($stmt);
		});
	}

	public static function removeFromUserFavorites($user, $post)
	{
		Database::transaction(function() use ($user, $post)
		{
			$post->removeCache('fav_count');
			$stmt = new Sql\DeleteStatement();
			$stmt->setTable('favoritee');
			$stmt->setCriterion((new Sql\ConjunctionFunctor)
				->add(new Sql\EqualsFunctor('post_id', new Sql\Binding($post->getId())))
				->add(new Sql\EqualsFunctor('user_id', new Sql\Binding($user->getId()))));
			Database::exec($stmt);
		});
	}

	public static function getAnonymousName()
	{
		return '[Anonymous user]';
	}

	public static function hashPassword($pass, $salt2)
	{
		$salt1 = Core::getConfig()->main->salt;
		return sha1($salt1 . $salt2 . $pass);
	}
}
